// const continuation = require('continuation-local-storage');
const continuation = require('./continuation-local-storage')
const requestify = require('requestify');
const uuid = require('uuid/v1');
const HttpClient = require('./client');

const NAMESPACE = 'ROBBYSON_HTTP_CLIENT_NAMESPACE';
const CONTEXT = 'ROBBYSON_HTTP_CLIENT_CONTEXT';

class RobbysonHttp {
    static setup (req, res, next) {
        let context;
        const session = continuation.createNamespace(NAMESPACE);
        session.run(() => {
            session.set(CONTEXT, new RobbysonHttp(req));
            context = session.active;
            next();
        });

        res.on('finish', () => {
            session.exit(context);
        });
    }

    constructor (req) {
        const sessionId = uuid();
        this.path = req.originalUrl;
        this.system_id = req.system_id;
        this.contractor_id = req.contractor_id;
        this.user = req.user || {};
        this.requestUUID = sessionId;
        req.requestUUID = sessionId;

        const headers = {};
        if (this.contractor_id) { headers['x-contractor-id'] = this.contractor_id; }
        if (this.user && this.user.sessionKey) { headers['x-robbyson-user'] = this.user.sessionKey; }
        if (this.system_id) { headers['x-system-id'] = this.system_id; }

        this.requestify = new HttpClient(headers);
    }

    static instance () {
        return continuation.getNamespace(NAMESPACE).get(CONTEXT);
    }

}

module.exports = RobbysonHttp;
