const contractorRepository = require('../repositories/contractor.repository');
const logRepository = require('../repositories/log.repository');
const resultRepository = require('../repositories/result.repository');
const typeRepository = require('../repositories/type.repository');

class Logger {
    constructor () {
        this.meta = {};
    }

    setRequestMetaData (meta) {
        this.meta = meta;
    }

    log (type, reason, data, result, system, contractorId = this.meta.contractorId) {
        if (!contractorId) {
            return false;
        }

        const typePromise = typeRepository.getByDescription(contractorId, type);
        const resultPromise = resultRepository.getByDescription(contractorId, result);
        system = { _id: parseInt(system) };
        const meta = this.meta;

        Promise.all([typePromise, resultPromise]).then(responses => {
            type = responses[0];
            result = responses[1];

            const log = {
                date: new Date(),
                ip: this.meta.clientIP,
                port: this.meta.clientPort,
                user: this.meta.user,
                data,
                reason,
                result,
                type,
                system
            };

            logRepository.save(contractorId, log);
        }).catch(error => console.error(error));
    }

    broadcast (type, reason, data, result, system) {
        contractorRepository.list().then(contractors => contractors.map(contractor => {
            this.log(type, reason, data, result, system, contractor._id);
        }));
    };

}

module.exports = Logger;
